@@ -4,7 +4,8 @@
 //! The materials define some of the inputs for the specified shader and provide additional configuration over the rendering pipeline such as alpha blending settings.
 //! The materials in the [Matl] file are assigned to objects in the [Mesh](crate::formats::mesh::Mesh) file by the [Modl](crate::formats::modl::Modl) file.
 
-use crate::{Color4f, SsbhString, Vector4};
+use crate::ssbh_enum;
+use crate::{Color4f, SsbhString, Vector4, Version};
 use crate::{SsbhArray, SsbhEnum64};
 use binread::BinRead;
 use ssbh_write::SsbhWrite;
@@ -20,7 +21,7 @@ use strum::{Display, EnumString, EnumVariantNames, FromRepr};
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite, PartialEq)]
-pub struct MatlAttributeV15 {
+pub struct AttributeV15 {
     /// Determines how the value in [param](#structfield.param) will be used by the shader.
     pub param_id: ParamId,
     /// The value and data type.
@@ -31,7 +32,7 @@ pub struct MatlAttributeV15 {
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite, PartialEq)]
-pub struct MatlAttributeV16 {
+pub struct AttributeV16 {
     /// Determines how the value in [param](#structfield.param) will be used by the shader.
     pub param_id: ParamId,
     /// The value and data type.
@@ -48,7 +49,7 @@ pub struct MatlEntryV15 {
     pub material_label: SsbhString,
 
     /// The collection of named material values.
-    pub attributes: SsbhArray<MatlAttributeV15>,
+    pub attributes: SsbhArray<AttributeV15>,
 
     /// The ID of the shader to associate with this material.
     /// For Smash Ultimate, the format is `<shader ID>_<render pass>`.
@@ -66,7 +67,7 @@ pub struct MatlEntryV16 {
     pub material_label: SsbhString,
 
     /// The collection of named material values.
-    pub attributes: SsbhArray<MatlAttributeV16>,
+    pub attributes: SsbhArray<AttributeV16>,
 
     /// The ID of the shader to associate with this material.
     /// For Smash Ultimate, the format is `<shader ID>_<render pass>`.
@@ -74,109 +75,69 @@ pub struct MatlEntryV16 {
     pub shader_label: SsbhString,
 }
 
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
-#[br(import(major_version: u16, minor_version: u16))]
-pub enum MatlEntries {
-    #[br(pre_assert(major_version == 1 &&  minor_version == 5))]
-    EntriesV15(SsbhArray<MatlEntryV15>),
-    #[br(pre_assert(major_version == 1 &&  minor_version == 6))]
-    EntriesV16(SsbhArray<MatlEntryV16>),
-}
-
 /// A container of materials.
 /// Compatible with file version 1.5 and 1.6.
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
-pub struct Matl {
-    pub major_version: u16,
-    pub minor_version: u16,
-    /// The material collection.
-    #[br(args(major_version, minor_version))]
-    pub entries: MatlEntries,
+#[derive(Debug, BinRead, SsbhWrite)]
+#[br(import(major_version: u16, minor_version: u16))]
+pub enum Matl {
+    #[br(pre_assert(major_version == 1 &&  minor_version == 5))]
+    V15 { entries: SsbhArray<MatlEntryV15> },
+    #[br(pre_assert(major_version == 1 &&  minor_version == 6))]
+    V16 { entries: SsbhArray<MatlEntryV16> },
 }
 
-/// A material parameter value.
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite, PartialEq)]
-#[br(import(data_type: u64))]
-pub enum ParamV15 {
-    #[br(pre_assert(data_type == 1u64))]
-    Float(f32),
-
-    #[br(pre_assert(data_type == 2u64))]
-    Boolean(u32),
+impl Version for Matl {
+    fn major_minor_version(&self) -> (u16, u16) {
+        match self {
+            Matl::V15 { .. } => (1, 5),
+            Matl::V16 { .. } => (1, 6),
+        }
+    }
+}
 
+ssbh_enum!(
+    /// A material parameter value.
+    ParamV15,
+    1u64 =>  Float(f32),
+    2u64 =>  Boolean(u32),
     /// A vector for storing RGBA colors, XYZW values, or up to four [f32] parameters.
-    #[br(pre_assert(data_type == 5u64))]
-    Vector4(Vector4),
-
+    5u64 =>  Vector4(Vector4),
     /// A vector for storing RGBA colors.
-    #[br(pre_assert(data_type == 7u64))]
-    Unk7(Color4f),
-
+    7u64 =>  Unk7(Color4f),
     /// A string value used to store texture file names.
     /// Examples: `"../../textures/cos_149000_02"`, `"/common/shader/sfxPBS/default_Params"`, `"#replace_cubemap"`, `"asf_ashley_col"`.
-    #[br(pre_assert(data_type == 11u64))]
-    MatlString(SsbhString),
-
-    #[br(pre_assert(data_type == 14u64))]
-    Sampler(MatlSampler),
-
-    #[br(pre_assert(data_type == 16u64))]
-    UvTransform(MatlUvTransform),
-
-    #[br(pre_assert(data_type == 17u64))]
-    BlendState(MatlBlendStateV15),
-
-    #[br(pre_assert(data_type == 18u64))]
-    RasterizerState(MatlRasterizerStateV15),
-}
-
-/// A material parameter value.
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite, PartialEq)]
-#[br(import(data_type: u64))]
-pub enum ParamV16 {
-    #[br(pre_assert(data_type == 1u64))]
-    Float(f32),
-
-    #[br(pre_assert(data_type == 2u64))]
-    Boolean(u32),
+    11u64 => String(SsbhString),
+    14u64 => Sampler(Sampler),
+    16u64 => UvTransform(UvTransform),
+    17u64 => BlendState(BlendStateV15),
+    18u64 => RasterizerState(RasterizerStateV15)
+);
 
+ssbh_enum!(
+    /// A material parameter value.
+    ParamV16,
+    1u64 =>  Float(f32),
+    2u64 =>  Boolean(u32),
     /// A vector for storing RGBA colors, XYZW values, or up to four [f32] parameters.
-    #[br(pre_assert(data_type == 5u64))]
-    Vector4(Vector4),
-
+    5u64 =>  Vector4(Vector4),
     /// A vector for storing RGBA colors.
-    #[br(pre_assert(data_type == 7u64))]
-    Unk7(Color4f),
-
+    7u64 =>  Unk7(Color4f),
     /// A string value used to store texture file names.
-    /// Examples: `"../../textures/cos_149000_02"`, `"/common/shader/sfxPBS/default_Params"`, `"#replace_cubemap"`, `"asf_ashley_col"`.    
-    #[br(pre_assert(data_type == 11u64))]
-    MatlString(SsbhString),
-
-    #[br(pre_assert(data_type == 14u64))]
-    Sampler(MatlSampler),
-
-    #[br(pre_assert(data_type == 16u64))]
-    UvTransform(MatlUvTransform),
-
-    #[br(pre_assert(data_type == 17u64))]
-    BlendState(MatlBlendStateV16),
-
-    #[br(pre_assert(data_type == 18u64))]
-    RasterizerState(MatlRasterizerStateV16),
-}
+    /// Examples: `"../../textures/cos_149000_02"`, `"/common/shader/sfxPBS/default_Params"`, `"#replace_cubemap"`, `"asf_ashley_col"`.
+    11u64 => String(SsbhString),
+    14u64 => Sampler(Sampler),
+    16u64 => UvTransform(UvTransform),
+    17u64 => BlendState(BlendStateV16),
+    18u64 => RasterizerState(RasterizerStateV16)
+);
 
 /// An enumeration of all possible material parameters.
 /// Not all values are used by Smash Ultimate's shaders.
-/// For up to date documentation, see the [Material Parameters](https://github.com/ScanMountGoat/Smush-Material-Research/blob/master/Material%20Parameters.md) page on Github.
+/// For up to date documentation, see the
+/// [Material Parameters](https://github.com/ScanMountGoat/Smush-Material-Research/blob/master/Material%20Parameters.md)
+/// page on Github.
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[cfg_attr(
@@ -585,13 +546,13 @@ pub enum FillMode {
 pub enum CullMode {
     Back = 0,
     Front = 1,
-    None = 2,
+    Disabled = 2,
 }
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, Clone, PartialEq, SsbhWrite)]
-pub struct MatlRasterizerStateV15 {
+pub struct RasterizerStateV15 {
     pub unk1: u32,
     pub unk2: u32,
 }
@@ -600,7 +561,7 @@ pub struct MatlRasterizerStateV15 {
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, Clone, PartialEq, SsbhWrite)]
 #[ssbhwrite(pad_after = 4)]
-pub struct MatlRasterizerStateV16 {
+pub struct RasterizerStateV16 {
     pub fill_mode: FillMode,
     pub cull_mode: CullMode,
     pub depth_bias: f32,
@@ -675,7 +636,7 @@ pub enum FilteringType {
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, Clone, PartialEq, SsbhWrite)]
-pub struct MatlSampler {
+pub struct Sampler {
     pub wraps: WrapMode,
     pub wrapt: WrapMode,
     pub wrapr: WrapMode,
@@ -689,6 +650,9 @@ pub struct MatlSampler {
     pub max_anisotropy: MaxAnisotropy,
 }
 
+/// Available anistropy levels for anisotropic texture filtering.
+///
+/// Higher values produce higher quality filtering at the cost of performance.
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(
     feature = "strum",
@@ -709,7 +673,7 @@ pub enum MaxAnisotropy {
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, Clone, PartialEq, SsbhWrite)]
-pub struct MatlUvTransform {
+pub struct UvTransform {
     pub x: f32, // TODO: this is probably the same as the anim data type
     pub y: f32,
     pub z: f32,
@@ -717,6 +681,7 @@ pub struct MatlUvTransform {
     pub v: f32,
 }
 
+/// Available blending modes for the source and destination color for alpha blending.
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(
     feature = "strum",
@@ -744,7 +709,7 @@ pub enum BlendFactor {
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, Clone, PartialEq, SsbhWrite)]
-pub struct MatlBlendStateV15 {
+pub struct BlendStateV15 {
     pub unk1: u64,
     pub unk2: u32,
     pub unk3: u32,
@@ -761,7 +726,7 @@ pub struct MatlBlendStateV15 {
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, Clone, PartialEq, SsbhWrite)]
 #[ssbhwrite(pad_after = 8)]
-pub struct MatlBlendStateV16 {
+pub struct BlendStateV16 {
     pub source_color: BlendFactor,
     pub unk2: u32,
     pub destination_color: BlendFactor,
