@@ -1,4 +1,5 @@
-use crate::{Color4f, InlineString, RelPtr64, Vector4};
+use crate::enums::ssbh_enum;
+use crate::{Color4f, InlineString, RelPtr64, Vector4, Version};
 use crate::{SsbhArray, SsbhEnum64, SsbhString};
 use binread::BinRead;
 #[cfg(feature = "serde")]
@@ -8,30 +9,18 @@ use ssbh_write::SsbhWrite;
 use super::matl::{BlendFactor, CullMode, FillMode, FilteringType, MagFilter, MinFilter, WrapMode};
 
 // TODO: Why are there slightly smaller variants?
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
-#[br(import(data_type: u64))]
-pub enum FrameBuffer {
-    #[br(pre_assert(data_type == 0u64))]
-    Framebuffer0(Framebuffer0),
-
-    #[br(pre_assert(data_type == 1u64))]
-    Framebuffer1(Framebuffer1),
-
-    #[br(pre_assert(data_type == 2u64))]
-    Framebuffer2(Framebuffer2),
-
-    #[br(pre_assert(data_type == 3u64))]
-    Framebuffer3(Framebuffer3),
-
-    #[br(pre_assert(data_type == 4u64))]
-    Framebuffer4(Framebuffer4),
-}
+ssbh_enum!(
+    FrameBuffer,
+    0u64 => Framebuffer0(Framebuffer0),
+    1u64 => Framebuffer1(Framebuffer1),
+    2u64 => Framebuffer2(Framebuffer2),
+    3u64 => Framebuffer3(Framebuffer3),
+    4u64 => Framebuffer4(Framebuffer4)
+);
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct Framebuffer0 {
     pub name: SsbhString,
     pub width: u32,
@@ -43,7 +32,7 @@ pub struct Framebuffer0 {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct Framebuffer1 {
     pub name: SsbhString,
     pub width: u32,
@@ -55,7 +44,7 @@ pub struct Framebuffer1 {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct Framebuffer2 {
     pub name: SsbhString,
     pub width: u32,
@@ -65,7 +54,7 @@ pub struct Framebuffer2 {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct Framebuffer3 {
     pub name: SsbhString,
     pub width: u32,
@@ -78,7 +67,7 @@ pub struct Framebuffer3 {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct Framebuffer4 {
     pub name: SsbhString,
     pub width: u32,
@@ -88,15 +77,15 @@ pub struct Framebuffer4 {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct FramebufferContainer {
     pub frame_buffer: SsbhEnum64<FrameBuffer>,
 }
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
-pub struct NrpdSampler {
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
+pub struct Sampler {
     pub name: SsbhString,
     pub wraps: WrapMode,
     pub wrapt: WrapMode,
@@ -114,8 +103,8 @@ pub struct NrpdSampler {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
-pub struct NrpdRasterizerState {
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
+pub struct RasterizerState {
     pub name: SsbhString,
     pub fill_mode: FillMode,
     pub cull_mode: CullMode,
@@ -127,8 +116,8 @@ pub struct NrpdRasterizerState {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
-pub struct NrpdBlendState {
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
+pub struct BlendState {
     pub name: SsbhString,
     pub source_color: BlendFactor,
     pub unk2: u32,
@@ -136,19 +125,17 @@ pub struct NrpdBlendState {
     pub unk4: u32,
     pub unk5: u32,
     pub unk6: u32,
-    pub unk7: u32,
+    pub alpha_sample_to_coverage: u32,
     pub unk8: u32,
     pub unk9: u32,
     pub unk10: u32,
 }
 
-/// A state type similar to `NrpdBlendState`.
-/// There is only a single instance of this struct,
-/// which make it's fields difficult to determine.
+// TODO: There is only a single instance of this struct?
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
-pub struct NrpdDepthState {
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
+pub struct DepthState {
     pub name: SsbhString,
     pub unk2: u32, // 4 booleans (1 byte each)?
     pub unk3: u32,
@@ -162,86 +149,44 @@ pub struct NrpdDepthState {
     pub unk11: u64,
 }
 
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
-#[br(import(data_type: u64))]
-pub enum NrpdState {
-    #[br(pre_assert(data_type == 0u64))]
-    Sampler(NrpdSampler),
-
-    #[br(pre_assert(data_type == 1u64))]
-    RasterizerState(NrpdRasterizerState),
-
-    #[br(pre_assert(data_type == 2u64))]
-    DepthState(NrpdDepthState),
-
-    #[br(pre_assert(data_type == 3u64))]
-    BlendState(NrpdBlendState),
-}
+ssbh_enum!(
+    State,
+    0u64 => Sampler(Sampler),
+    1u64 => RasterizerState(RasterizerState),
+    2u64 => DepthState(DepthState),
+    3u64 => BlendState(BlendState)
+);
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct StateContainer {
-    pub state: SsbhEnum64<NrpdState>,
+    pub state: SsbhEnum64<State>,
 }
 
 // TODO: The variant names are just guesses based on the string values.
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
-#[br(import(data_type: u64))]
-pub enum RenderPassData {
-    #[br(pre_assert(data_type == 0u64))]
-    FramebufferRtp(RenderPassData0),
-
-    #[br(pre_assert(data_type == 1u64))]
-    Depth(u64), // TODO
-
-    #[br(pre_assert(data_type == 2u64))]
-    UnkTexture1(RenderPassData2),
-
-    #[br(pre_assert(data_type == 3u64))]
-    UnkLight(RenderPassData3),
-
-    #[br(pre_assert(data_type == 8u64))]
-    Unk8(RenderPassData8),
-
-    #[br(pre_assert(data_type == 9u64))]
-    ColorClear(RenderPassData9),
-
-    #[br(pre_assert(data_type == 10u64))]
-    DepthStencilClear(RenderPassData10),
-
-    #[br(pre_assert(data_type == 12u64))]
-    Viewport(RenderPassData12),
-
-    #[br(pre_assert(data_type == 13u64))]
-    Sampler(RenderPassData13),
-
-    #[br(pre_assert(data_type == 14u64))]
-    BlendState(StringPair),
-
-    #[br(pre_assert(data_type == 15u64))]
-    RasterizerState(StringPair),
-
-    #[br(pre_assert(data_type == 16u64))]
-    DepthStencilState(StringPair),
-
-    #[br(pre_assert(data_type == 17u64))]
-    FramebufferRenderTarget(SsbhString),
-
-    #[br(pre_assert(data_type == 18u64))]
-    FramebufferDepthStencil(SsbhString),
-
-    #[br(pre_assert(data_type == 19u64))]
-    UnkTexture2(u64), // TODO
-}
+ssbh_enum!(
+    RenderPassData,
+    0u64 =>  FramebufferRtp(RenderPassData0),
+    1u64 =>  Depth(u64), // TODO
+    2u64 =>  UnkTexture1(RenderPassData2),
+    3u64 =>  UnkLight(RenderPassData3),
+    8u64 =>  Unk8(RenderPassData8),
+    9u64 =>  ColorClear(RenderPassData9),
+    10u64 => DepthStencilClear(RenderPassData10),
+    12u64 => Viewport(RenderPassData12),
+    13u64 => Sampler(RenderPassData13),
+    14u64 => BlendState(StringPair),
+    15u64 => RasterizerState(StringPair),
+    16u64 => DepthStencilState(StringPair),
+    17u64 => FramebufferRenderTarget(SsbhString),
+    18u64 => FramebufferDepthStencil(SsbhString),
+    19u64 => UnkTexture2(u64) // TODO
+);
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct RenderPassData0 {
     unk1: SsbhString,
     unk2: SsbhString,
@@ -250,7 +195,7 @@ pub struct RenderPassData0 {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct RenderPassData2 {
     unk1: SsbhString,
     unk2: SsbhString,
@@ -262,7 +207,7 @@ pub struct RenderPassData2 {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct RenderPassData3 {
     unk1: SsbhString,
     unk2: SsbhString,
@@ -274,7 +219,7 @@ pub struct RenderPassData3 {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct RenderPassData8 {
     unk1: SsbhString,
     unk2: SsbhString,
@@ -286,7 +231,7 @@ pub struct RenderPassData8 {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct Unk8Data {
     unk1: u32,
     unk2: u32,
@@ -294,7 +239,7 @@ pub struct Unk8Data {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct RenderPassData9 {
     unk1: SsbhString,
     unk2: Vector4,
@@ -303,7 +248,7 @@ pub struct RenderPassData9 {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct RenderPassData10 {
     unk1: SsbhString,
     unk2: f32,
@@ -312,7 +257,7 @@ pub struct RenderPassData10 {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct RenderPassData12 {
     unk1: SsbhString,
     unk2: u64,
@@ -322,7 +267,7 @@ pub struct RenderPassData12 {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct RenderPassData13 {
     unk1: SsbhString,
     unk2: SsbhString,
@@ -335,7 +280,7 @@ pub struct RenderPassData13 {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 #[ssbhwrite(pad_after = 8)]
 pub struct RenderPassContainer {
     pub name: SsbhString,
@@ -345,21 +290,15 @@ pub struct RenderPassContainer {
     pub unk3: SsbhEnum64<RenderPassUnkData>,
 }
 
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
-#[br(import(data_type: u64))]
-pub enum RenderPassUnkData {
-    #[br(pre_assert(data_type == 0u64))]
-    Unk0(InlineString),
-
-    #[br(pre_assert(data_type == 3u64))]
-    Unk3(Unk3Data),
-}
+ssbh_enum!(
+    RenderPassUnkData,
+    0 => Unk0(InlineString),
+    3 => Unk3(Unk3Data)
+);
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct Unk3Data {
     pub name: SsbhString,
     pub unk1: f32,
@@ -370,7 +309,7 @@ pub struct Unk3Data {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct StringPair {
     pub item1: SsbhString,
     pub item2: SsbhString,
@@ -378,7 +317,7 @@ pub struct StringPair {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct UnkItem1 {
     pub unk1: SsbhString,
     pub unk2: SsbhArray<UnkItem3>,
@@ -386,7 +325,7 @@ pub struct UnkItem1 {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct UnkItem2 {
     pub unk1: RelPtr64<StringPair>,
     pub unk2: u64,
@@ -394,7 +333,7 @@ pub struct UnkItem2 {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
 pub struct UnkItem3 {
     pub name: SsbhString,
     pub value: SsbhString,
@@ -404,27 +343,37 @@ pub struct UnkItem3 {
 /// Compatible with file version 1.6.
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, SsbhWrite)]
-pub struct Nrpd {
-    pub major_version: u16,
-    pub minor_version: u16,
-    pub frame_buffer_containers: SsbhArray<FramebufferContainer>,
-    pub state_containers: SsbhArray<StateContainer>,
-    // TODO: The data pointer is too small after writing the render_passes.
-    pub render_passes: SsbhArray<RenderPassContainer>,
-    pub unk_string_list1: SsbhArray<StringPair>,
-    pub unk_string_list2: SsbhArray<UnkItem2>,
-    pub unk_list: SsbhArray<UnkItem1>,
-    pub unk_width1: u32,
-    pub unk_height1: u32,
-    pub unk3: u32,
-    pub unk4: u32,
-    pub unk5: u32,
-    pub unk6: u32, // flags?
-    pub unk7: u32,
-    pub unk8: u32,
-    pub unk9: SsbhString, // empty string?
-    pub unk_width2: u32,
-    pub unk_height2: u32,
-    pub unk10: u64,
+#[derive(Debug, BinRead, SsbhWrite, PartialEq)]
+#[br(import(major_version: u16, minor_version: u16))]
+pub enum Nrpd {
+    #[br(pre_assert(major_version == 1 && minor_version == 6))]
+    V16 {
+        frame_buffer_containers: SsbhArray<FramebufferContainer>,
+        state_containers: SsbhArray<StateContainer>,
+        // TODO: The data pointer is too small after writing the render_passes.
+        render_passes: SsbhArray<RenderPassContainer>,
+        unk_string_list1: SsbhArray<StringPair>,
+        unk_string_list2: SsbhArray<UnkItem2>,
+        unk_list: SsbhArray<UnkItem1>,
+        unk_width1: u32,
+        unk_height1: u32,
+        unk3: u32,
+        unk4: u32,
+        unk5: u32,
+        unk6: u32, // flags?
+        unk7: u32,
+        unk8: u32,
+        unk9: SsbhString, // empty string?
+        unk_width2: u32,
+        unk_height2: u32,
+        unk10: u64,
+    },
+}
+
+impl Version for Nrpd {
+    fn major_minor_version(&self) -> (u16, u16) {
+        match self {
+            Nrpd::V16 { .. } => (1, 6),
+        }
+    }
 }
